@@ -1,3 +1,5 @@
+/* -*- mode:C++; tab-width:8; c-basic-offset:8; indent-tabs-mode:true -*- */
+
 /*
  * written by J. Marcin Gorycki <marcin.gorycki@intel.com>
  *
@@ -20,27 +22,45 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <games-gconf.h>
+#include <games-frame.h>
+
 #include "scoreframe.h"
 
+// FIXME: This is also defined in tetris.cpp
+#define KEY_STARTING_LEVEL "/apps/gnometris/options/starting_level"
+
 ScoreFrame::ScoreFrame(int cmdlLevel)
 	: score(0), lines(0)
 {
-  startingLevel = cmdlLevel ? cmdlLevel : gnome_config_get_int_with_default("/gnometris/Properties/StartingLevel=1", 0);
+	if (cmdlLevel) 
+		startingLevel = cmdlLevel;
+	else {
+		GConfClient *gconf_client = gconf_client_get_default ();
+		startingLevel = gconf_client_get_int (gconf_client,
+						      KEY_STARTING_LEVEL,
+						      NULL);
+		g_object_unref (gconf_client);
+	}
+	if (startingLevel < 1)
+		startingLevel = 1;
+
 	level = startingLevel;
 	
-	w = gtk_frame_new(_("Game Status"));
+	w = games_frame_new (_("Game Status"));
+	games_frame_set (GAMES_FRAME (w), 0);
 
-	scoreLabel = gtk_label_new(_("Score: "));
+	scoreLabel = gtk_label_new (_("Score:"));
 	sprintf(b, "%7d", 0);
-	scorew = gtk_label_new(b);
+	scorew = gtk_label_new (b);
 
-	linesLabel = gtk_label_new(_("Lines: "));
+	linesLabel = gtk_label_new (_("Lines:"));
 	sprintf(b, "%7d", 0);
-	linesw = gtk_label_new(b);
+	linesw = gtk_label_new (b);
 
-	levelLabel = gtk_label_new(_("Level: "));
+	levelLabel = gtk_label_new (_("Level:"));
 	sprintf(b, "%7d", level);
-	levelw = gtk_label_new(b);
+	levelw = gtk_label_new (b);
 
 	vb = gtk_vbox_new(FALSE, 0);
 	hbScore = gtk_hbox_new(FALSE, 0);
@@ -67,17 +87,7 @@ ScoreFrame::ScoreFrame(int cmdlLevel)
 void
 ScoreFrame::show()
 {
-	gtk_widget_show(scoreLabel);
-	gtk_widget_show(linesLabel);
-	gtk_widget_show(levelLabel);
-	gtk_widget_show(scorew);
-	gtk_widget_show(linesw);
-	gtk_widget_show(levelw);
-	gtk_widget_show(hbScore);
-	gtk_widget_show(hbLines);
-	gtk_widget_show(hbLevel);
-	gtk_widget_show(vb);
-	gtk_widget_show(w);
+	gtk_widget_show_all (w);
 }
 
 void 
